@@ -35,6 +35,31 @@ lexbor_str_init(lexbor_str_t *str, lexbor_mraw_t *mraw, size_t size)
     return str->data;
 }
 
+lxb_char_t *
+lexbor_str_init_append(lexbor_str_t *str, lexbor_mraw_t *mraw,
+                       const lxb_char_t *data, size_t length)
+{
+    lxb_char_t *p;
+
+    if (str == NULL) {
+        return NULL;
+    }
+
+    p = lexbor_mraw_alloc(mraw, (length + 1));
+    if (p == NULL) {
+        return NULL;
+    }
+
+    memcpy(p, data, length);
+
+    p[length] = '\0';
+
+    str->data = p;
+    str->length = length;
+
+    return p;
+}
+
 void
 lexbor_str_clean(lexbor_str_t *str)
 {
@@ -55,6 +80,7 @@ lexbor_str_destroy(lexbor_str_t *str, lexbor_mraw_t *mraw, bool destroy_obj)
     }
 
     if (str->data != NULL) {
+        lexbor_str_clean(str);
         str->data = lexbor_mraw_free(mraw, str->data);
     }
 
@@ -108,6 +134,10 @@ lexbor_str_append(lexbor_str_t *str, lexbor_mraw_t *mraw,
 {
     lxb_char_t *data_begin;
 
+    if (length == 0) {
+        return str->data;
+    }
+
     lexbor_str_check_size_arg_m(str, lexbor_str_size(str),
                                 mraw, (length + 1), NULL);
 
@@ -351,6 +381,30 @@ lexbor_str_whitespace_from_end(lexbor_str_t *target)
     return 0;
 }
 
+lxb_char_t *
+lexbor_str_copy_to(lexbor_str_t *str, const lxb_char_t *buff, size_t length)
+{
+    lxb_char_t *data_begin;
+
+    data_begin = &str->data[str->length];
+    memcpy(data_begin, buff, sizeof(lxb_char_t) * length);
+
+    str->length += length;
+
+    return data_begin;
+}
+
+lxb_char_t *
+lexbor_str_copy_to_with_null(lexbor_str_t *str,
+                             const lxb_char_t *buff, size_t length)
+{
+    lxb_char_t *data_begin = lexbor_str_copy_to(str, buff, length);
+
+    str->data[str->length] = '\0';
+
+    return data_begin;
+}
+
 /*
  * Data utils
  * TODO: All functions need optimization.
