@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2018 Alexander Borisov
+ * Copyright (C) 2018-2025 Alexander Borisov
  *
  * Author: Alexander Borisov <borisov@lexbor.com>
  */
@@ -16,8 +16,6 @@ extern "C" {
 #include "lexbor/tag/const.h"
 #include "lexbor/ns/const.h"
 
-#include "lexbor/dom/exception.h"
-
 
 #define lxb_dom_interface_cdata_section(obj) ((lxb_dom_cdata_section_t *) (obj))
 #define lxb_dom_interface_character_data(obj) ((lxb_dom_character_data_t *) (obj))
@@ -61,6 +59,10 @@ typedef lxb_dom_interface_t *
 (*lxb_dom_interface_create_f)(lxb_dom_document_t *document, lxb_tag_id_t tag_id,
                               lxb_ns_id_t ns);
 
+typedef lxb_dom_interface_t *
+(*lxb_dom_interface_clone_f)(lxb_dom_document_t *document,
+                             const lxb_dom_interface_t *intrfc);
+
 typedef lxb_dom_interface_t *
 (*lxb_dom_interface_destroy_f)(lxb_dom_interface_t *intrfc);
 
@@ -69,6 +71,10 @@ LXB_API lxb_dom_interface_t *
 lxb_dom_interface_create(lxb_dom_document_t *document, lxb_tag_id_t tag_id,
                          lxb_ns_id_t ns);
 
+LXB_API lxb_dom_interface_t *
+lxb_dom_interface_clone(lxb_dom_document_t *document,
+                        const lxb_dom_interface_t *intrfc);
+
 LXB_API lxb_dom_interface_t *
 lxb_dom_interface_destroy(lxb_dom_interface_t *intrfc);
 
