@@ -19,6 +19,7 @@
 #include "lexbor/html/interface.h"
 #include "lexbor/html/interfaces/template_element.h"
 #include "lexbor/html/interfaces/unknown_element.h"
+#include "lexbor/html/interfaces/option_element.h"
 #include "lexbor/html/tokenizer/state_rawtext.h"
 #include "lexbor/html/tokenizer/state_rcdata.h"
 
@@ -390,7 +391,8 @@ lxb_html_tree_appropriate_place_inserting_node(lxb_html_tree_t *tree,
 
 lxb_html_element_t *
 lxb_html_tree_insert_foreign_element(lxb_html_tree_t *tree,
-                                     lxb_html_token_t *token, lxb_ns_id_t ns)
+                                     lxb_html_token_t *token, lxb_ns_id_t ns,
+                                     bool only_add_stack)
 {
     lxb_status_t status;
     lxb_dom_node_t *pos;
@@ -398,20 +400,16 @@ lxb_html_tree_insert_foreign_element(lxb_html_tree_t *tree,
     lxb_html_tree_insertion_position_t ipos;
 
     pos = lxb_html_tree_appropriate_place_inserting_node(tree, NULL, &ipos);
-
-    if (ipos == LXB_HTML_TREE_INSERTION_POSITION_CHILD) {
-        element = lxb_html_tree_create_element_for_token(tree, token, ns, pos);
-    }
-    else {
-        element = lxb_html_tree_create_element_for_token(tree, token, ns,
-                                                         pos->parent);
+    if (pos == NULL) {
+        return NULL;
     }
 
+    element = lxb_html_tree_create_element_for_token(tree, token, ns);
     if (element == NULL) {
         return NULL;
     }
 
-    if (pos != NULL) {
+    if (only_add_stack == false) {
         lxb_html_tree_insert_node(pos, lxb_dom_interface_node(element), ipos);
     }
 
@@ -426,8 +424,7 @@ lxb_html_tree_insert_foreign_element(lxb_html_tree_t *tree,
 
 lxb_html_element_t *
 lxb_html_tree_create_element_for_token(lxb_html_tree_t *tree,
-                                       lxb_html_token_t *token, lxb_ns_id_t ns,
-                                       lxb_dom_node_t *parent)
+                                       lxb_html_token_t *token, lxb_ns_id_t ns)
 {
     lxb_dom_node_t *node = lxb_html_tree_create_node(tree, token->tag_id, ns);
     if (node == NULL) {
@@ -535,7 +532,7 @@ lxb_html_tree_append_attributes_from_element(lxb_html_tree_t *tree,
             }
         }
 
-        lxb_dom_element_attr_append(element, attr);
+        lxb_dom_element_attr_append(element, new_attr);
 
         attr = attr->next;
     }
@@ -1029,44 +1026,7 @@ lxb_html_tree_reset_insertion_mode_appropriately(lxb_html_tree_t *tree)
             continue;
         }
 
-        /* Step 4 */
-        if (node->local_name == LXB_TAG_SELECT) {
-            /* Step 4.1 */
-            if (last) {
-                tree->mode = lxb_html_tree_insertion_mode_in_select;
-                return;
-            }
-
-            /* Step 4.2 */
-            size_t ancestor = idx;
-
-            for (;;) {
-                /* Step 4.3 */
-                if (ancestor == 0) {
-                    tree->mode = lxb_html_tree_insertion_mode_in_select;
-                    return;
-                }
-
-                /* Step 4.4 */
-                ancestor--;
-
-                /* Step 4.5 */
-                lxb_dom_node_t *ancestor_node = list[ancestor];
-
-                if(lxb_html_tree_node_is(ancestor_node, LXB_TAG_TEMPLATE)) {
-                    tree->mode = lxb_html_tree_insertion_mode_in_select;
-                    return;
-                }
-
-                /* Step 4.6 */
-                else if(lxb_html_tree_node_is(ancestor_node, LXB_TAG_TABLE)) {
-                    tree->mode = lxb_html_tree_insertion_mode_in_select_in_table;
-                    return;
-                }
-            }
-        }
-
-        /* Step 5-15 */
+        /* Step 4-15 */
         switch (node->local_name) {
             case LXB_TAG_TD:
             case LXB_TAG_TH:
@@ -1305,6 +1265,41 @@ lxb_html_tree_element_in_scope_td_th(lxb_html_tree_t *tree)
     return NULL;
 }
 
+lxb_dom_node_t *
+lxb_html_tree_element_in_scope_option_optgroup(lxb_html_tree_t *tree)
+{
+    lxb_dom_node_t *node;
+
+    size_t idx = tree->open_elements->length;
+    void **list = tree->open_elements->list;
+
+    while (idx != 0) {
+        idx--;
+        node = list[idx];
+
+        switch (node->local_name) {
+            case LXB_TAG_OPTION:
+            case LXB_TAG_OPTGROUP:
+                if (node->ns == LXB_NS_HTML) {
+                    return node;
+                }
+
+                break;
+
+            default:
+                break;
+        }
+
+        if (lxb_html_tag_is_category(node->local_name, LXB_NS_HTML,
+                                     LXB_HTML_TAG_CATEGORY_SCOPE))
+        {
+            return NULL;
+        }
+    }
+
+    return NULL;
+}
+
 bool
 lxb_html_tree_check_scope_element(lxb_html_tree_t *tree)
 {
@@ -1549,8 +1544,7 @@ lxb_html_tree_adoption_agency_algorithm(lxb_html_tree_t *tree,
             fake_token.base_element = node;
 
             element = lxb_html_tree_create_element_for_token(tree, &fake_token,
-                                                             LXB_NS_HTML,
-                                                             common_ancestor);
+                                                             LXB_NS_HTML);
             if (element == NULL) {
                 *status = LXB_STATUS_ERROR_MEMORY_ALLOCATION;
 
@@ -1571,7 +1565,7 @@ lxb_html_tree_adoption_agency_algorithm(lxb_html_tree_t *tree,
 
             /* State 14.9 */
             if (last->parent != NULL) {
-                lxb_dom_node_remove(last);
+                lxb_dom_node_remove_wo_events(last);
             }
 
             lxb_dom_node_insert_child_wo_events(node, last);
@@ -1581,7 +1575,7 @@ lxb_html_tree_adoption_agency_algorithm(lxb_html_tree_t *tree,
         }
 
         if (last->parent != NULL) {
-            lxb_dom_node_remove(last);
+            lxb_dom_node_remove_wo_events(last);
         }
 
         /* State 15 */
@@ -1604,8 +1598,7 @@ lxb_html_tree_adoption_agency_algorithm(lxb_html_tree_t *tree,
         fake_token.base_element = formatting_element;
 
         element = lxb_html_tree_create_element_for_token(tree, &fake_token,
-                                                         LXB_NS_HTML,
-                                                         furthest_block);
+                                                         LXB_NS_HTML);
         if (element == NULL) {
             *status = LXB_STATUS_ERROR_MEMORY_ALLOCATION;
 
@@ -1619,7 +1612,7 @@ lxb_html_tree_adoption_agency_algorithm(lxb_html_tree_t *tree,
         while (node != NULL) {
             next = node->next;
 
-            lxb_dom_node_remove(node);
+            lxb_dom_node_remove_wo_events(node);
             lxb_dom_node_insert_child_wo_events(lxb_dom_interface_node(element),
                                                 node);
             node = next;
