@@ -111,12 +111,12 @@ lxb_html_tree_appropriate_place_inserting_node(lxb_html_tree_t *tree,
 
 LXB_API lxb_html_element_t *
 lxb_html_tree_insert_foreign_element(lxb_html_tree_t *tree,
-                                     lxb_html_token_t *token, lxb_ns_id_t ns);
+                                     lxb_html_token_t *token, lxb_ns_id_t ns,
+                                     bool only_add_stack);
 
 LXB_API lxb_html_element_t *
 lxb_html_tree_create_element_for_token(lxb_html_tree_t *tree,
-                                       lxb_html_token_t *token, lxb_ns_id_t ns,
-                                       lxb_dom_node_t *parent);
+                                       lxb_html_token_t *token, lxb_ns_id_t ns);
 
 LXB_API lxb_status_t
 lxb_html_tree_append_attributes(lxb_html_tree_t *tree,
@@ -199,6 +199,9 @@ lxb_html_tree_element_in_scope_tbody_thead_tfoot(lxb_html_tree_t *tree);
 LXB_API lxb_dom_node_t *
 lxb_html_tree_element_in_scope_td_th(lxb_html_tree_t *tree);
 
+LXB_API lxb_dom_node_t *
+lxb_html_tree_element_in_scope_option_optgroup(lxb_html_tree_t *tree);
+
 LXB_API bool
 lxb_html_tree_check_scope_element(lxb_html_tree_t *tree);
 
@@ -275,7 +278,7 @@ lxb_html_tree_create_node(lxb_html_tree_t *tree,
 }
 
 lxb_inline bool
-lxb_html_tree_node_is(lxb_dom_node_t *node, lxb_tag_id_t tag_id)
+lxb_html_tree_node_is(const lxb_dom_node_t *node, lxb_tag_id_t tag_id)
 {
     return node->local_name == tag_id && node->ns == LXB_NS_HTML;
 }
@@ -305,7 +308,8 @@ lxb_inline lxb_html_element_t *
 lxb_html_tree_insert_html_element(lxb_html_tree_t *tree,
                                   lxb_html_token_t *token)
 {
-    return lxb_html_tree_insert_foreign_element(tree, token, LXB_NS_HTML);
+    return lxb_html_tree_insert_foreign_element(tree, token, LXB_NS_HTML,
+                                                false);
 }
 
 lxb_inline void
@@ -331,7 +335,7 @@ lxb_html_tree_acknowledge_token_self_closing(lxb_html_tree_t *tree,
 
     bool is_void = lxb_html_tag_is_void(token->tag_id);
 
-    if (is_void) {
+    if (!is_void) {
         lxb_html_tree_parse_error(tree, token,
                                   LXB_HTML_RULES_ERROR_NOVOHTELSTTAWITRSO);
     }
@@ -372,61 +376,21 @@ lxb_html_tree_attach_document(lxb_html_tree_t *tree, lxb_html_document_t *doc)
     tree->document = doc;
 }
 
-/*
- * No inline functions for ABI.
- */
-LXB_API lxb_status_t
-lxb_html_tree_begin_noi(lxb_html_tree_t *tree, lxb_html_document_t *document);
+lxb_inline bool
+lxb_html_tree_is_fragment(lxb_html_tree_t *tree)
+{
+    return tree->fragment != NULL;
+}
 
-LXB_API lxb_status_t
-lxb_html_tree_chunk_noi(lxb_html_tree_t *tree, const lxb_char_t *html,
-                        size_t size);
-
-LXB_API lxb_status_t
-lxb_html_tree_end_noi(lxb_html_tree_t *tree);
-
-LXB_API lxb_status_t
-lxb_html_tree_build_noi(lxb_html_tree_t *tree, lxb_html_document_t *document,
-                        const lxb_char_t *html, size_t size);
-
-LXB_API lxb_dom_node_t *
-lxb_html_tree_create_node_noi(lxb_html_tree_t *tree,
-                              lxb_tag_id_t tag_id, lxb_ns_id_t ns);
-
-LXB_API bool
-lxb_html_tree_node_is_noi(lxb_dom_node_t *node, lxb_tag_id_t tag_id);
-
-LXB_API lxb_dom_node_t *
-lxb_html_tree_current_node_noi(lxb_html_tree_t *tree);
-
-LXB_API lxb_dom_node_t *
-lxb_html_tree_adjusted_current_node_noi(lxb_html_tree_t *tree);
-
-LXB_API lxb_html_element_t *
-lxb_html_tree_insert_html_element_noi(lxb_html_tree_t *tree,
-                                      lxb_html_token_t *token);
-
-LXB_API void
-lxb_html_tree_insert_node_noi(lxb_dom_node_t *to, lxb_dom_node_t *node,
-                              lxb_html_tree_insertion_position_t ipos);
-
-LXB_API void
-lxb_html_tree_acknowledge_token_self_closing_noi(lxb_html_tree_t *tree,
-                                             lxb_html_token_t *token);
-
-LXB_API bool
-lxb_html_tree_mathml_text_integration_point_noi(lxb_dom_node_t *node);
-
-LXB_API bool
-lxb_html_tree_scripting_noi(lxb_html_tree_t *tree);
-
-LXB_API void
-lxb_html_tree_scripting_set_noi(lxb_html_tree_t *tree, bool scripting);
-
-LXB_API void
-lxb_html_tree_attach_document_noi(lxb_html_tree_t *tree,
-                                  lxb_html_document_t *doc);
+lxb_inline bool
+lxb_html_tree_is_fragment_element(lxb_html_tree_t *tree,
+                                  lxb_tag_id_t tag_id, lxb_ns_id_t ns)
+{
+    lxb_dom_node_t *fragment = tree->fragment;
 
+    return lxb_html_tree_is_fragment(tree)
+            && fragment->local_name == tag_id && fragment->ns == ns;
+}
 
 #ifdef __cplusplus
 } /* extern "C" */
