@@ -9,10 +9,23 @@
 #include <lexbor/core/avl.h>
 
 
+typedef struct {
+    size_t remove;
+    size_t *result;
+    size_t *p;
+}
+avl_test_ctx_t;
+
+
+static lxb_status_t
+avl_cb(lexbor_avl_t *avl, lexbor_avl_node_t **root,
+       lexbor_avl_node_t *node, void *ctx);
+
+
 TEST_BEGIN(init)
 {
     lexbor_avl_t *avl = lexbor_avl_create();
-    lxb_status_t status = lexbor_avl_init(avl, 1024);
+    lxb_status_t status = lexbor_avl_init(avl, 1024, 0);
 
     test_eq(status, LXB_STATUS_OK);
 
@@ -22,7 +35,7 @@ TEST_END
 
 TEST_BEGIN(init_null)
 {
-    lxb_status_t status = lexbor_avl_init(NULL, 1024);
+    lxb_status_t status = lexbor_avl_init(NULL, 1024, 0);
     test_eq(status, LXB_STATUS_ERROR_OBJECT_IS_NULL);
 }
 TEST_END
@@ -30,7 +43,7 @@ TEST_END
 TEST_BEGIN(init_stack)
 {
     lexbor_avl_t avl;
-    lxb_status_t status = lexbor_avl_init(&avl, 1024);
+    lxb_status_t status = lexbor_avl_init(&avl, 1024, 0);
 
     test_eq(status, LXB_STATUS_OK);
 
@@ -41,7 +54,7 @@ TEST_END
 TEST_BEGIN(init_args)
 {
     lexbor_avl_t avl = {0};
-    lxb_status_t status = lexbor_avl_init(&avl, 0);
+    lxb_status_t status = lexbor_avl_init(&avl, 0, 0);
 
     test_eq(status, LXB_STATUS_ERROR_WRONG_ARGS);
 
@@ -52,7 +65,7 @@ TEST_END
 TEST_BEGIN(node_make)
 {
     lexbor_avl_t avl;
-    test_eq(lexbor_avl_init(&avl, 1024), LXB_STATUS_OK);
+    test_eq(lexbor_avl_init(&avl, 1024, 0), LXB_STATUS_OK);
 
     lexbor_avl_node_t *node = lexbor_avl_node_make(&avl, 1,
                                                    &avl);
@@ -72,7 +85,7 @@ TEST_END
 TEST_BEGIN(node_clean)
 {
     lexbor_avl_t avl;
-    test_eq(lexbor_avl_init(&avl, 1024), LXB_STATUS_OK);
+    test_eq(lexbor_avl_init(&avl, 1024, 0), LXB_STATUS_OK);
 
     lexbor_avl_node_t *node = lexbor_avl_node_make(&avl, 1,
                                                    &avl);
@@ -101,7 +114,7 @@ TEST_END
 TEST_BEGIN(node_destroy)
 {
     lexbor_avl_t *avl = lexbor_avl_create();
-    lexbor_avl_init(avl, 1024);
+    lexbor_avl_init(avl, 1024, 0);
 
     lexbor_avl_node_t *node = lexbor_avl_node_make(avl, 1,
                                                    &avl);
@@ -168,7 +181,7 @@ TEST_BEGIN(tree_3_0)
     lexbor_avl_t avl;
     lexbor_avl_node_t *root = NULL;
 
-    test_eq(lexbor_avl_init(&avl, 1024), LXB_STATUS_OK);
+    test_eq(lexbor_avl_init(&avl, 1024, 0), LXB_STATUS_OK);
 
     lexbor_avl_insert(&avl, &root, 1, (void *) 1);
     lexbor_avl_insert(&avl, &root, 2, (void *) 2);
@@ -185,7 +198,7 @@ TEST_BEGIN(tree_3_1)
     lexbor_avl_t avl;
     lexbor_avl_node_t *root = NULL;
 
-    test_eq(lexbor_avl_init(&avl, 1024), LXB_STATUS_OK);
+    test_eq(lexbor_avl_init(&avl, 1024, 0), LXB_STATUS_OK);
 
     lexbor_avl_insert(&avl, &root, 3, (void *) 3);
     lexbor_avl_insert(&avl, &root, 2, (void *) 2);
@@ -202,7 +215,7 @@ TEST_BEGIN(tree_3_2)
     lexbor_avl_t avl;
     lexbor_avl_node_t *root = NULL;
 
-    test_eq(lexbor_avl_init(&avl, 1024), LXB_STATUS_OK);
+    test_eq(lexbor_avl_init(&avl, 1024, 0), LXB_STATUS_OK);
 
     lexbor_avl_insert(&avl, &root, 2, (void *) 2);
     lexbor_avl_insert(&avl, &root, 1, (void *) 1);
@@ -219,7 +232,7 @@ TEST_BEGIN(tree_3_3)
     lexbor_avl_t avl;
     lexbor_avl_node_t *root = NULL;
 
-    test_eq(lexbor_avl_init(&avl, 1024), LXB_STATUS_OK);
+    test_eq(lexbor_avl_init(&avl, 1024, 0), LXB_STATUS_OK);
 
     lexbor_avl_insert(&avl, &root, 2, (void *) 2);
     lexbor_avl_insert(&avl, &root, 3, (void *) 3);
@@ -236,7 +249,7 @@ TEST_BEGIN(tree_3_4)
     lexbor_avl_t avl;
     lexbor_avl_node_t *root = NULL;
 
-    test_eq(lexbor_avl_init(&avl, 1024), LXB_STATUS_OK);
+    test_eq(lexbor_avl_init(&avl, 1024, 0), LXB_STATUS_OK);
 
     lexbor_avl_insert(&avl, &root, 3, (void *) 3);
     lexbor_avl_insert(&avl, &root, 1, (void *) 1);
@@ -253,7 +266,7 @@ TEST_BEGIN(tree_3_5)
     lexbor_avl_t avl;
     lexbor_avl_node_t *root = NULL;
 
-    test_eq(lexbor_avl_init(&avl, 1024), LXB_STATUS_OK);
+    test_eq(lexbor_avl_init(&avl, 1024, 0), LXB_STATUS_OK);
 
     lexbor_avl_insert(&avl, &root, 1, (void *) 1);
     lexbor_avl_insert(&avl, &root, 3, (void *) 3);
@@ -270,7 +283,7 @@ TEST_BEGIN(tree_4)
     lexbor_avl_t avl;
     lexbor_avl_node_t *root = NULL, *node;
 
-    test_eq(lexbor_avl_init(&avl, 1024), LXB_STATUS_OK);
+    test_eq(lexbor_avl_init(&avl, 1024, 0), LXB_STATUS_OK);
 
     lexbor_avl_insert(&avl, &root, 1, (void *) 1);
     lexbor_avl_insert(&avl, &root, 2, (void *) 2);
@@ -333,7 +346,7 @@ TEST_BEGIN(tree_5)
     lexbor_avl_t avl;
     lexbor_avl_node_t *root = NULL, *node;
 
-    test_eq(lexbor_avl_init(&avl, 1024), LXB_STATUS_OK);
+    test_eq(lexbor_avl_init(&avl, 1024, 0), LXB_STATUS_OK);
 
     lexbor_avl_insert(&avl, &root, 1, (void *) 1);
     lexbor_avl_insert(&avl, &root, 2, (void *) 2);
@@ -409,7 +422,7 @@ TEST_BEGIN(delete_1L)
     lexbor_avl_t avl;
     lexbor_avl_node_t *root = NULL, *node;
 
-    test_eq(lexbor_avl_init(&avl, 1024), LXB_STATUS_OK);
+    test_eq(lexbor_avl_init(&avl, 1024, 0), LXB_STATUS_OK);
 
     lexbor_avl_insert(&avl, &root, 1, (void *) 1);
     lexbor_avl_insert(&avl, &root, 2, (void *) 2);
@@ -464,7 +477,7 @@ TEST_BEGIN(delete_1R)
     lexbor_avl_t avl;
     lexbor_avl_node_t *root = NULL, *node;
 
-    test_eq(lexbor_avl_init(&avl, 1024), LXB_STATUS_OK);
+    test_eq(lexbor_avl_init(&avl, 1024, 0), LXB_STATUS_OK);
 
     lexbor_avl_insert(&avl, &root, 3, (void *) 3);
     lexbor_avl_insert(&avl, &root, 2, (void *) 2);
@@ -519,7 +532,7 @@ TEST_BEGIN(delete_2L)
     lexbor_avl_t avl;
     lexbor_avl_node_t *root = NULL, *node;
 
-    test_eq(lexbor_avl_init(&avl, 1024), LXB_STATUS_OK);
+    test_eq(lexbor_avl_init(&avl, 1024, 0), LXB_STATUS_OK);
 
     lexbor_avl_insert(&avl, &root, 2, (void *) 2);
     lexbor_avl_insert(&avl, &root, 1, (void *) 1);
@@ -574,7 +587,7 @@ TEST_BEGIN(delete_2R)
     lexbor_avl_t avl;
     lexbor_avl_node_t *root = NULL, *node;
 
-    test_eq(lexbor_avl_init(&avl, 1024), LXB_STATUS_OK);
+    test_eq(lexbor_avl_init(&avl, 1024, 0), LXB_STATUS_OK);
 
     lexbor_avl_insert(&avl, &root, 3, (void *) 3);
     lexbor_avl_insert(&avl, &root, 1, (void *) 1);
@@ -629,7 +642,7 @@ TEST_BEGIN(delete_sub_1L)
     lexbor_avl_t avl;
     lexbor_avl_node_t *root = NULL, *node;
 
-    test_eq(lexbor_avl_init(&avl, 1024), LXB_STATUS_OK);
+    test_eq(lexbor_avl_init(&avl, 1024, 0), LXB_STATUS_OK);
 
     lexbor_avl_insert(&avl, &root, 3, (void *) 3);
     lexbor_avl_insert(&avl, &root, 2, (void *) 2);
@@ -725,7 +738,7 @@ TEST_BEGIN(delete_sub_1R)
     lexbor_avl_t avl;
     lexbor_avl_node_t *root = NULL, *node;
 
-    test_eq(lexbor_avl_init(&avl, 1024), LXB_STATUS_OK);
+    test_eq(lexbor_avl_init(&avl, 1024, 0), LXB_STATUS_OK);
 
     lexbor_avl_insert(&avl, &root, 5, (void *) 5);
     lexbor_avl_insert(&avl, &root, 3, (void *) 3);
@@ -822,7 +835,7 @@ TEST_BEGIN(delete_10_0)
     lexbor_avl_t avl;
     lexbor_avl_node_t *root = NULL, *node;
 
-    test_eq(lexbor_avl_init(&avl, 1024), LXB_STATUS_OK);
+    test_eq(lexbor_avl_init(&avl, 1024, 0), LXB_STATUS_OK);
 
     lexbor_avl_insert(&avl, &root, 1, (void *) 1);
     lexbor_avl_insert(&avl, &root, 2, (void *) 2);
@@ -869,7 +882,7 @@ TEST_BEGIN(delete_10_1)
     lexbor_avl_t avl;
     lexbor_avl_node_t *root = NULL, *node;
 
-    test_eq(lexbor_avl_init(&avl, 1024), LXB_STATUS_OK);
+    test_eq(lexbor_avl_init(&avl, 1024, 0), LXB_STATUS_OK);
 
     lexbor_avl_insert(&avl, &root, 1, (void *) 1);
     lexbor_avl_insert(&avl, &root, 2, (void *) 2);
@@ -917,7 +930,7 @@ TEST_BEGIN(delete_10_2)
     lexbor_avl_t avl;
     lexbor_avl_node_t *root = NULL, *node;
 
-    test_eq(lexbor_avl_init(&avl, 1024), LXB_STATUS_OK);
+    test_eq(lexbor_avl_init(&avl, 1024, 0), LXB_STATUS_OK);
 
     lexbor_avl_insert(&avl, &root, 1, (void *) 1);
     lexbor_avl_insert(&avl, &root, 2, (void *) 2);
@@ -965,7 +978,7 @@ TEST_BEGIN(delete_10_3)
     lexbor_avl_t avl;
     lexbor_avl_node_t *root = NULL, *node;
 
-    test_eq(lexbor_avl_init(&avl, 1024), LXB_STATUS_OK);
+    test_eq(lexbor_avl_init(&avl, 1024, 0), LXB_STATUS_OK);
 
     lexbor_avl_insert(&avl, &root, 1, (void *) 1);
     lexbor_avl_insert(&avl, &root, 2, (void *) 2);
@@ -1012,7 +1025,7 @@ TEST_BEGIN(delete_10_4)
     lexbor_avl_t avl;
     lexbor_avl_node_t *root = NULL, *node;
 
-    test_eq(lexbor_avl_init(&avl, 1024), LXB_STATUS_OK);
+    test_eq(lexbor_avl_init(&avl, 1024, 0), LXB_STATUS_OK);
 
     lexbor_avl_insert(&avl, &root, 1, (void *) 1);
     lexbor_avl_insert(&avl, &root, 2, (void *) 2);
@@ -1059,7 +1072,7 @@ TEST_BEGIN(delete_10_5)
     lexbor_avl_t avl;
     lexbor_avl_node_t *root = NULL, *node;
 
-    test_eq(lexbor_avl_init(&avl, 1024), LXB_STATUS_OK);
+    test_eq(lexbor_avl_init(&avl, 1024, 0), LXB_STATUS_OK);
 
     lexbor_avl_insert(&avl, &root, 1, (void *) 1);
     lexbor_avl_insert(&avl, &root, 2, (void *) 2);
@@ -1104,7 +1117,7 @@ TEST_END
 TEST_BEGIN(clean)
 {
     lexbor_avl_t avl;
-    lexbor_avl_init(&avl, 1024);
+    lexbor_avl_init(&avl, 1024, 0);
 
     lexbor_avl_clean(&avl);
 
@@ -1115,12 +1128,12 @@ TEST_END
 TEST_BEGIN(destroy)
 {
     lexbor_avl_t *avl = lexbor_avl_create();
-    lexbor_avl_init(avl, 1024);
+    lexbor_avl_init(avl, 1024, 0);
 
     test_eq(lexbor_avl_destroy(avl, true), NULL);
 
     avl = lexbor_avl_create();
-    lexbor_avl_init(avl, 1021);
+    lexbor_avl_init(avl, 1021, 0);
 
     test_eq(lexbor_avl_destroy(avl, false), avl);
     test_eq(lexbor_avl_destroy(avl, true), NULL);
@@ -1131,12 +1144,143 @@ TEST_END
 TEST_BEGIN(destroy_stack)
 {
     lexbor_avl_t avl;
-    lexbor_avl_init(&avl, 1023);
+    lexbor_avl_init(&avl, 1023, 0);
 
     test_eq(lexbor_avl_destroy(&avl, false), &avl);
 }
 TEST_END
 
+TEST_BEGIN(foreach_4)
+{
+    size_t i, *p;
+    lexbor_avl_t avl;
+    avl_test_ctx_t test;
+    lexbor_avl_node_t *root = NULL;
+
+    test_eq(lexbor_avl_init(&avl, 1024, 0), LXB_STATUS_OK);
+
+    for (i = 5; i > 1; i--) {
+        lexbor_avl_insert(&avl, &root, i, NULL);
+    }
+
+    test.result = lexbor_malloc(10 * sizeof(size_t));
+    test_ne(test.result, NULL);
+
+    test.remove = 4;
+    test.p = test.result;
+
+    lexbor_avl_foreach(&avl, &root, avl_cb, &test);
+
+    p = test.result;
+
+    for (i = 2; i < 6; i++) {
+        test_ne(p, test.p);
+
+        test_eq(i, *p);
+
+        p++;
+    }
+
+    lexbor_free(test.result);
+    lexbor_avl_destroy(&avl, false);
+}
+TEST_END
+
+TEST_BEGIN(foreach_6)
+{
+    size_t i, *p;
+    lexbor_avl_t avl;
+    avl_test_ctx_t test;
+    lexbor_avl_node_t *root = NULL;
+
+    test_eq(lexbor_avl_init(&avl, 1024, 0), LXB_STATUS_OK);
+
+    for (i = 5; i < 9; i++) {
+        lexbor_avl_insert(&avl, &root, i, NULL);
+    }
+
+    test.result = lexbor_malloc(10 * sizeof(size_t));
+    test_ne(test.result, NULL);
+
+    test.remove = 6;
+    test.p = test.result;
+
+    lexbor_avl_foreach(&avl, &root, avl_cb, &test);
+
+    p = test.result;
+
+    for (i = 5; i < 9; i++) {
+        test_ne(p, test.p);
+
+        test_eq(i, *p);
+
+        p++;
+    }
+
+    lexbor_free(test.result);
+    lexbor_avl_destroy(&avl, false);
+}
+TEST_END
+
+TEST_BEGIN(foreach_10)
+{
+    size_t i, *p;
+    lexbor_avl_t avl;
+    avl_test_ctx_t test;
+    lexbor_avl_node_t *root;
+
+    static const size_t total = 101;
+
+    test.result = lexbor_malloc(total * sizeof(size_t));
+    test_ne(test.result, NULL);
+
+    for (size_t r = 1; r < total; r++) {
+        test_eq(lexbor_avl_init(&avl, 1024, 0), LXB_STATUS_OK);
+
+        root = NULL;
+
+        for (i = 1; i < total; i++) {
+            lexbor_avl_insert(&avl, &root, i, NULL);
+        }
+
+        test.remove = r;
+        test.p = test.result;
+
+        lexbor_avl_foreach(&avl, &root, avl_cb, &test);
+
+        p = test.result;
+
+        for (i = 1; i < total; i++) {
+            test_ne(p, test.p);
+
+            test_eq(i, *p);
+
+            p++;
+        }
+
+        lexbor_avl_destroy(&avl, false);
+    }
+
+    lexbor_free(test.result);
+}
+TEST_END
+
+static lxb_status_t
+avl_cb(lexbor_avl_t *avl, lexbor_avl_node_t **root,
+       lexbor_avl_node_t *node, void *ctx)
+{
+    avl_test_ctx_t *test = ctx;
+
+    *test->p = node->type;
+    test->p++;
+
+    if (node->type == test->remove) {
+        lexbor_avl_remove_by_node(avl, root, node);
+    }
+
+    return LXB_STATUS_OK;
+}
+
 int
 main(int argc, const char * argv[])
 {
@@ -1173,6 +1317,10 @@ main(int argc, const char * argv[])
     TEST_ADD(destroy);
     TEST_ADD(destroy_stack);
 
+    TEST_ADD(foreach_4);
+    TEST_ADD(foreach_6);
+    TEST_ADD(foreach_10);
+
     TEST_RUN("lexbor/core/avl");
     TEST_RELEASE();
 }
