@@ -17,9 +17,9 @@ import LXB
 attributes_name = [
     "id", "class", "dir", "width", "height", "pool", "href", "alt", "disabled", "src",
     "style", "title", "checked", "maxlength", "content", "http-equiv", "scheme",
-    "charser", "is", "for", "slot", "html", "system", "public", "type", "color",
+    "charset", "is", "for", "slot", "html", "system", "public", "type", "color",
     "face", "size", "active", "selected", "focus", "hover", "required", "placeholder",
-    "readonly"
+    "readonly", "multiple"
 ]
 
 
